=head1 NAME

 oss

=head1 PREFACE

 This package is the public perl API to configure the OpenSchoolServer.

=head1 SYNOPSIS

 #!/usr/bin/perl 
 
 BEGIN{ push @INC,"/usr/share/oss/lib/"; }
 
 use oss;
 
 my $uid           = shift;
 my $userpassword  = shift;
 my $remote        = shift || '';
 my $oid           = shift || '';
 my $dn;
 my $reply;

 my $oss = oss->new();
 
 $oss->connect_ldap() || die ($oss->{ERROR}->{text});
 
 if( $oid ne '' ) {
   my $sdn = $oss->get_school_base($oid);
   $dn = $oss->get_user_dn($uid,$sdn);
 } else {
   $dn = $oss->get_user_dn($uid);
 }
 
 if($remote eq '' ) {
   $reply = $oss->login($dn,$userpassword,0);
 } else {
   $reply = $oss->login($dn,$userpassword,1,$remote);
 }
 
 if( $reply ) {
   print $oss->reply($reply);
 } else {
   die ($oss->{ERROR}->{text});
 }

=head1 DESCRIPTION

B<oss>  is a collection of functions that implement a OpenSchoolServer 
configuration API for Perl programs.

=over 2

=cut

BEGIN{ 
  push @INC,"/usr/share/oss/lib/"; 
}

package oss;

use strict;
use oss_utils;
use Net::LDAP;
use Net::LDAP::Util qw( ldap_error_name ldap_error_text) ;
use Net::LDAP::Entry;
use Net::IMAP;
use Net::Netmask;
use Socket;
use Time::Local;
use Digest::MD5  qw(md5_hex);
use ManageSieve;
use MIME::Base64;
use Crypt::SmbHash;
use Storable qw(thaw freeze);
# The standard output format is XML
# Debug only
use Data::Dumper;

#######################################################################################
# End header									      #
#######################################################################################

#######################################################################################
# Basic Utilities								      #
#######################################################################################

=item B<new()>

The Constructor of an oss object

EXAMPLE:

  my $oss = oss->new();

=cut

sub new
{
    my ($this, $LDAP_CONF)     = @_;
    my $class    = ref($this) || $this;
    my $self     = {};
    $self->{XML} = 1;
    $self->{DEFAULT_USER_ATTRIBUTES} = [
                'admin',
                'birthday',
                'c',
		'cn',
                'facsimiletelephonenumber',
                'fquota',
                'gidnumber',
                'givenname',
                'group',
                'homephone',
                'jpegphoto',
                'l',
                'labeleduri',
                'mail',
                'mobile',
                'ou',
                'oxtimezone',
                'postalcode',
                'preferredlanguage',
                'quota',
                'religion',
                'role',
                'sn',
                'st',
                'street',
                'susemailacceptaddress',
                'telephonenumber',
                'title',
                'uid',
                'uidnumber'

	];
    $self->{DEFAULT_GROUP_ATTRIBUTES} = [
                'cn',
                'deliverytofolder',
                'deliverytomember',
                'description',
                'fquota',
                'gidNumber',
                'grouptype',
                'member',
                'quota',
                'role',
                'susemailforwardaddress',
                'writerdn'
	];
    if( defined $LDAP_CONF && $LDAP_CONF ne '' )
    {
      $self->{LDAP_CONF} = $LDAP_CONF;
    }
    else
    {
      $self->{LDAP_CONF} = '/etc/ldap.conf';
    }
    

    bless $self, $class;
    return $self;
}
#-----------------------------------------------------------------------

=item B<destroy()>

The destructor of an oss object

EXAMPLE:

  my $oss = oss->destroy();

=cut

sub destroy
{
    my $this   = shift;
    if( defined $this->{LDAP} )
    {
      $this->{LDAP}->unbind();
    }
    if( defined $this->{IMAP} )
    {
      $this->{IMAP}->logout();
    }
    $this = {};
    return 1;
}
#-----------------------------------------------------------------------

=item B<connect_imap(user, userpassword)>

Make an IMAP connection.

EXAMPLE:

  connect_imap();

  Makes an IMAP-connection as user C<cyrus>. Works only if the program will be executed with root rights.

  connect_ldap('varkpete','12345678');

  Makes an IMAP-connection as user C<varkpete>. 

=cut

sub connect_imap()
{
    my $this     = shift;
    my $user     = shift || 'cyrus';
    my $password = shift || '';

    if( $user eq 'admin' )
    {
	$user = 'cyrus';
    }
    if( $user eq 'cyrus' )
    {
      $this->init_admin();
      $password = $this->{APW};
    }
    if( !defined $this->{SYSCONFIG}->{SCHOOL_MAILSERVER} )
    {
      $this->init_sysconfig('file'); 
    }

    $this->{IMAP} = new Net::IMAP($this->{SYSCONFIG}->{SCHOOL_MAILSERVER}, Debug => 0); 
    if( ! $this->{IMAP} )
    {
      $this->{ERROR}->{text} = "ERROR Could not make IMAP connection to $this->{SYSCONFIG}->{SCHOOL_MAILSERVER}\n";
      return undef;
    }
    if( ! $this->{IMAP}->login($user, $password) )
    {
      $this->{ERROR}->{text} = "ERROR Could not login to the IMAP server $this->{SYSCONFIG}->{SCHOOL_MAILSERVER} as $user\n";
      return undef;
    }
    return 1;
}
#-----------------------------------------------------------------------

=item B<connect_ldap(type, bind_dn, userpassword)>

Initialize the global variable $LDAP.

EXAMPLE:

  connect_ldap('anon');

  Makes an anonymous LDAP-Connection

  connect_ldap('admin');

  Makes an  LDAP-Connection with root-DN rigts. Works only if the programm
  will be executed with root rights. In this case the subroutine <init_admin>
  will be called.

  connect_ldap('user','uid=varkpete,ou=people,dc=extis,dc=de','12345678');

  Makes an LDAP-Connection with user-DN rigts. 
   
=cut

sub connect_ldap
{
    my $this     = shift;
    my $type     = shift || 'admin';
    my $dn       = shift || '';
    my $password = shift || '';

    if( defined $this->{LDAP} )
    {
      $this->{ERROR}->{text} = "ERROR LDAP Already connected\n";
      return -1;
    }

    # Searching for LDAP-Server settings
    my ($LDAP_BASE, $LDAP_SERVER, $LDAP_PORT) = parse_file($this->{LDAP_CONF}, "BASE", "HOST", "PORT");
    $LDAP_BASE =~ s/\s*?^(.*)\s*$/$1/i;
    if($LDAP_BASE eq "")
    {
        $this->{ERROR}->{text} = "ERROR Unable to parse ldap.conf or baseDN was not found!\n";
	return 0;
    }
    if($LDAP_SERVER eq "")
    {
        $LDAP_SERVER = "localhost";
    }
    if($LDAP_PORT eq "") 
    {
        $LDAP_PORT = 389;
    }
    $this->{LDAP} = Net::LDAP->new($LDAP_SERVER,port => $LDAP_PORT ,version => 3);
    if( ! $this->{LDAP} )
    {
        $this->{ERROR}->{text} = "Could not make LDAP connection\n";
	return 0;
    }
    if( $type ne 'anon' )
    {
      if( $type eq 'admin' )
      {
        $this->init_admin();
      }
      if( $type eq 'user' )
      {
        $this->{BIND_DN} = $dn;
	$this->{APW}     = $password;
      }
      if( ! $this->{LDAP}->bind($this->{BIND_DN}, password=>$this->{APW}) )
      {
        $this->{ERROR}->{text} = "Could not bind LDAP\n";
	return 0;
      }
    }
    $this->{LDAP_BASE}   = $LDAP_BASE;
    $this->{LDAP_SERVER} = $LDAP_SERVER;
    $this->{LDAP_PORT}   = $LDAP_PORT;
    return 1;
}
#-----------------------------------------------------------------------

=item B<connect_sieve(|user)>

Creates an sieve connection. If user is given, a proxy connection will be made.
If not, an admin connection will be made.

EXAMPLE:

  my $oss = oss->new;
  $oss->connect_sieve('micmou');

  Makes a sieve proxy connection as micmou. 
  
  $oss->connect_sieve();

  Makes a sieve connection as cyrus. 
  These both works only if the script will be executed as root.
  
  $oss->connect_sieve('micmou','12345mimou');
  
  Makes a sieve connection as micmou. 

  $oss->{SIEVE}->deleteScript('filter.sieve');

=cut

sub connect_sieve()
{
    my $this     = shift;
    my $user     = shift  || 'cyrus';
    my $password = shift  || '';
    my $port     = 2000;

    if( $user eq 'cyrus' && !defined $this->{APW} )
    {
      $this->init_admin();
    }

    if( !defined $this->{SYSCONFIG}->{SCHOOL_MAILSERVER} )
    {
      $this->init_sysconfig(); 
    }
    if( defined $this->{SYSCONFIG}->{SCHOOL_SIEVEPORT} )
    {
      $port = $this->{SYSCONFIG}->{SCHOOL_SIEVEPORT};
    }

    $this->{SIEVE} = ManageSieve->new($this->{SYSCONFIG}->{SCHOOL_MAILSERVER}  ,$port);
    if( ! $this->{SIEVE}->connect )
    {
      $this->{ERROR}->{text} = "ERROR Could not make SIEVE connection to $this->{SYSCONFIG}->{SCHOOL_MAILSERVER}\n";
      return undef;
    }
    if( $password eq '' )
    {
      if( ! $this->{SIEVE}->authplain($user, 'cyrus', $this->{APW}) )
      {
        $this->{ERROR}->{text} = "ERROR Could not login to the IMAP server $this->{SYSCONFIG}->{SCHOOL_MAILSERVER} as $user\n";
        return undef;
      }
    }
    else
    {
      if( ! $this->{SIEVE}->authplain($user, $user, $password ))
      {
        $this->{ERROR}->{text} = "ERROR Could not login to the IMAP server $this->{SYSCONFIG}->{SCHOOL_MAILSERVER} as $user\n";
        return undef;
      }
    }  
}
#-----------------------------------------------------------------------

=item B<delete_ldap_children(dn)>

Delets the children of an LDAP-entry

EXAMPLE:

  delete_ldap_children('uid=micmou,ou=people,dc=schule,dc=de');

=cut

sub delete_ldap_children($)
{
    my $this = shift;
    my $dn = shift;

    my $mesg = $this->{LDAP}->search( base   => $dn,
    			              scope  => 'one',
			              filter => 'objectclass=*',
			              attrs  => [ 'dn' ]
			            );
    if( $mesg->code )
    {
      $this->ldap_error($mesg);
      return 0;
    }
    foreach my $entry ($mesg->entries)
    {
      if( $this->delete_ldap_children( $entry->dn()) )
      {
        $this->{LDAP}->delete($entry->dn());
      }
      else
      {
        return 0;
      }
    }
    return 1;

}
#-----------------------------------------------------------------------

=item B<exists_dn(dn)>

Returns 1 if a dn exists and 0 if not.

EXAMPLE:

   my $user_exist = exists_dn($dn); 

=cut

sub exists_dn($)
{
  my $this   = shift;
  my $dn     = shift;

  my $mesg = $this->{LDAP}->search( base   => $dn, 
                                    scope  => 'base', 
				    filter => 'objectclass=*',
				    attrs  => []
				  );
  if( $mesg->code || ! $mesg->count )
  {
    $this->{ERROR} = "'$dn' do not exists.";
    return 0;
  }
  return 1;

}
#-----------------------------------------------------------------------

=item B<get_entries_dn(filter,school_dn)>

Delivers the dns of the result of an LDAP search

   my @templates = get_entries_dn("(role=templates)");

=cut

sub get_entries_dn
{
    my $this        = shift;
    my $filter      = shift;
    my $school_base = shift || $this->{LDAP_BASE};
    my @dn          = ();

    my $mesg = $this->{LDAP}->search( base   => $school_base,
    				      filter => $filter,
				      scope  => 'sub',
				      attrs  => [ 'dn' ]
				    );  
    foreach my $entry ( $mesg->entries() )
    {
      push @dn, $entry->dn();
    }
    return @dn;
}

#-----------------------------------------------------------------------

=item B<get_entry(DN)>

Returns an Net::LDAP::Entry object of a DN

EXAMPLE:

 my $entry = get_entry(DN-of-the-object);

=cut

sub get_entry($)
{
    my $this = shift;
    my $dn   = shift;

    my $mesg = $this->{LDAP}->search( base   => $dn,
                                      scope  => 'base',
                                      filter => '(objectclass=*)'
                            );
    if( $mesg && $mesg->count() == 1)
    {
      return $mesg->entry(0);
    }
    return undef;
}

#-----------------------------------------------------------------------

=item B<get_config_value>

Returns an configurations value of a object

EXAMPLE:

 my $VALUE = get_config_value('KEY',DN-of-the-object);

=cut

sub get_config_value($$)
{
    my $this = shift;
    my $key  = shift;
    my $dn   = shift;

    my $mesg = $this->{LDAP}->search( base   => $dn,
                                      scope  => 'base',
                                      filter => '(objectclass=*)',
                                      attrs  => [ 'configurationValue' ]
                            );
    if( $mesg && $mesg->count() == 1)
    {
      my @values = $mesg->entry(0)->get_value('configurationValue');
      foreach my $config (@values)
      {
        my ($keyn,$value) = split /=/,$config,2;
        if( $keyn =~ /^$key$/i )
        {
          return $value;
        }
      }
    }
    return undef;
}

#-----------------------------------------------------------------------

=item B<get_attribute(dn,attribute)>

Returns an attribute of the object given by the dn. 

EXAMPLE:

  my $homedirectory = get_attribute('uid=varkpete,ou=people,dc=extis,dc=de','homedirectory');

=cut

sub get_attribute($$)
{
    my $this      = shift;
    my $dn        = shift;
    my $attribute = shift;

  my $mesg = $this->{LDAP}->search( base  => $dn,
                            scope => 'base',
			    filter=> '(objectclass=*)',
		            attrs => [ $attribute ]
  		);
  if( $mesg->code() || $mesg->count() != 1 )
  {
    return undef;
  }
  return $mesg->entry(0)->get_value($attribute);
}
#-----------------------------------------------------------------------

=item B<get_attributes(dn,attributes)>

Returns attributes of the object given by the dn. The second parameter is a 
pointerr to a array.

EXAMPLE:

  my $attributes = get_attributes('uid=varkpete,ou=people,dc=extis,dc=de',['homedirectory','uidnumber','gidnumber]);

  print $attributes->{homedirectory}."\n";
  print $attributes->{uidnumber}."\n";
  print $attributes->{gidnumber}."\n";

=cut

sub get_attributes($$)
{
  my $this = shift;
  my $dn         = shift;
  my $attributes = shift;
  my %reply      = ();

  my $mesg = $this->{LDAP}->search( base  => $dn,
  		           scope => 'base',
		           attrs => $attributes
  		);
  if( $mesg->code() || $mesg->count() != 1 )
  {
    return undef;
  }
  foreach my $attribute ( @{$attributes} )
  {
     push @{$reply{$attribute}}, $mesg->entry(0)->get_value($attribute);
  }
  return \%reply;
}
#-----------------------------------------------------------------------

=item B<get_next_unique(user|group)>

Returns the next free unique identifier

EXAMPLE:

  my $uidnumber = $oss->get_next_unique(user);

=cut

sub get_next_unique
{
    my $this = shift;
    my $type = shift || 'user';
    my $base = 'cn='.$type.'configuration,ou=ldapconfig,'.$this->{LDAP_BASE};

    #TODO at the time we only count up;
    my $oldid = $this->get_attribute( $base,'suseNextUniqueid');

    while ( !$this->is_unique($oldid,$type) )
    {
	$oldid ++;
    }
    my $newid = $oldid +1;
    my $mesg = $this->{LDAP}->modify( $base,
		           replace =>  { suseNextUniqueid => "$newid" }
			);
    return $oldid;
}
#-----------------------------------------------------------------------

=item B<init_admin()>

Initialize the global variable BIND_DN, APW and LDAP_BASE with the
system settings. Works only if the programm will be executed with
root rights.

EXAMPLE:

  init_admin();

  print $BIND_DN;

=cut

sub init_admin
{
    my $this = shift;
    $this->{BIND_DN} =`. /etc/sysconfig/ldap ; echo -n \$BIND_DN;`;
    $this->{APW}     =`/usr/sbin/oss_get_admin_pw`;
}    
#-----------------------------------------------------------------------

=item B<init_sysconfig('file'|dn)>

Reads the configuration of a school into the global hash %this->{SYSCONFIG}->.

EXAMPLE:

  init_sysconfig('file');

  Reads the variables from the file "/etc/sysconfig/schoolserver".

  init_sysconfig();

  Reads the variables from LDAP-Server with base "ou=sysconfig,$this->{LDAP_BASE}".

  init_sysconfig('uniqueIdentifier=1008,dc=realschule-bayern,dc=info');

  Reads the variables of the school with the dn from LDAP-Server.


=cut

sub init_sysconfig 
{
    my $this = shift;
    # TODO das ist zur zeit nur so, eigentlich sollten die ldap-objekte
    # immer geben.
    my $dn = shift;

    my $base = $this->{LDAP_BASE}; 

    if( defined $dn && $dn eq 'file')
    {
      open( IN, "/etc/sysconfig/schoolserver" );
      while(<IN>){
        next if (/^#/);
        /([A-Z_]+)="(.*)"/;
        if($1)
	{
          $this->{SYSCONFIG}->{$1}= $2;
        }
      }
      $this->{SYSCONFIG}->{DHCP_BASE}       = 'ou=DHCP,'.$this->{LDAP_BASE};
      $this->{SYSCONFIG}->{DNS_BASE}        = 'ou=DNS,'.$this->{LDAP_BASE};
      $this->{SYSCONFIG}->{COMPUTERS_BASE}  = 'ou=Computers,'.$this->{LDAP_BASE};
      $this->{SYSCONFIG}->{USER_BASE}       = 'ou=people,'.$this->{LDAP_BASE};
      $this->{SYSCONFIG}->{GROUP_BASE}      = 'ou=group,'.$this->{LDAP_BASE};
      return 1;
    }
    if( defined $dn && $dn ne '' )
    {
	$base = $dn;
    }
    my $mesg = $this->{LDAP}->search( base   => 'ou=sysconfig,'.$base,
                              scope  => 'one',
		              filter => '(&(objectClass=SchoolConfiguration)(configurationKey=SCHOOL_*))'
		 );
    if( $mesg->code() || $mesg->count() < 1 )
    {
       $this->{ERROR}->{text}  = "ERROR Can not read the school configuration for dn=$dn\n";
       return 0;
    }
    foreach my $entry ( $mesg->entries() )
    {
      $this->{SYSCONFIG}->{$entry->get_value('configurationKey')} = $entry->get_value('configurationValue');
    }
    return 1;
}
#-----------------------------------------------------------------------

=item B<is_unique(id,user|group)>

Returns the next free unique identifier

EXAMPLE:

  my $is_unique = $oss->is_unique(1076,'user');

=cut

sub is_unique
{
    my $this = shift;
    my $id   = shift;
    my $type = shift || 'user';
    my $attribute = 'uidnumber';
    my $oc        = 'posixaccount';

    if( $type eq 'group')
    {
       $attribute = 'gidnumber';
       $oc        = 'posixgroup';
    }

    my $mesg = $this->{LDAP}->search(
       			base   => $this->{LDAP_BASE},
			filter => "($attribute=$id)",
			attrs  => []
    );

    if( $mesg->count )
    {
      return 0;
    }

    return 1;
}
#-----------------------------------------------------------------------

=item B<reply>

Returns a list of dn hasches in  XML or text form depending on the value of $this->{XML}

EXAMPLE:

 print $oss->reply($users);

=cut

sub reply
{
    my $this = shift;
    my $result = shift;
    my $out    = '';

    if( $this->{XML} )
    {
      $out = reply_xml(hash_to_xml($result));
    }
    else 
    {
      $out = hash_to_text($result);
    }
    return $out;
}

#-----------------------------------------------------------------------

=item B<reply>

Returns a dn in  XML or text form depending on the value of $this->{XML}

EXAMPLE:

 print $oss->reply($users);

=cut

sub replydn
{
    my $this = shift;
    my $dn = shift;
    my $out    = '';

    if( $this->{XML} )
    {
      $out    = "<reply>\n";
      $out .= "  <dn>".$dn."</dn>\n";  
      $out .= "</reply>\n";
    }
    else
    {
      $out = $dn;
    }
    return $out;
}
#-----------------------------------------------------------------------

=item B<ldap_error(mesg)>

Evaluates the return value of an ldap operation and puts in the $this->{ERROR} hash.

EXAMPLE:

    my $mesg = $this->{LDAP}->search( base   => $this->{LDAP_BASE},
			      filter => "(&(objectClass=posixAccount)(uid=$uid))",
			      scope  => 'sub',
			      attrs  => [ 'uidNumber' ]
    );
    if( $mesg->code() ){
       $this->ldap_error($mesg);
       print $this->{ERROR}->{text};
       return undef;
    }
    return $mesg->entry(0)->get_value('uidNumber'); 


=cut

sub ldap_error($)
{
    my $this = shift;
    my $mesg = shift;

    $this->{ERROR}->{text}  = " LDAP-Server return code: ".$mesg->code ;
    $this->{ERROR}->{text} .= " Message: ".ldap_error_name($mesg->code);
    $this->{ERROR}->{text} .= "\n :".ldap_error_text($mesg->code);
    $this->{ERROR}->{text} .= "\n";

    $this->{ERROR}->{code}  = $mesg->code;
}
#-----------------------------------------------------------------------

#######################################################################################
# Utilities to manipulate the users						      #
#######################################################################################

=item B<add_user_to_group(userDN,groupDN)>

Delets an user from a group. Delets the acls of the on the shared mailbox too.

EXAMPLE:

 $oss->add_user_to_group('uid=micmou,ou=people,dc=schule,dc=de','cn=5A,ougroup,dc=schule,dc=de');

=cut

sub add_user_to_group($$)
{
    my $this = shift;
    my $udn  = shift;
    my $gdn  = shift;

    my $gidnumber   = $this->get_attribute($gdn,'gidnumber');
    my $uid         = get_name_of_dn($udn);

    $this->{IMAP}->setacl(get_name_of_dn($gdn),$uid);
    $this->{LDAP}->modify($gdn, add=> { member    => $udn }); 
    $this->{LDAP}->modify($gdn, add=> { memberUID => $uid }); 
    $this->{LDAP}->modify($udn, add=> { OXGroupID => $gidnumber });
}
#-----------------------------------------------------------------------

=item B<disable_user(dn)>

Disable an user to login.

EXAMPLE:

  disable_user('uid=micmou,ou=people,dc=schule,dc=de');

=cut

sub disable_user($)
{
    my $this = shift;
    my $dn   = shift;

    $this->{LDAP}->modify( $dn , replace => { sambaAcctFlags => '[UD         ]'} );
    $this->{LDAP}->modify( $dn , delete  => 'shadowExpire' );
    $this->{LDAP}->modify( $dn , add     => { shadowExpire => 1 });
}
#-----------------------------------------------------------------------

=item B<enable_user(dn)>

Enable an user to login.

EXAMPLE:

 enable_user('uid=micmou,ou=people,dc=schule,dc=de');

=cut

sub enable_user($)
{
    my $this = shift;
    my $dn = shift;

    $this->{LDAP}->modify( $dn , replace => { sambaAcctFlags => '[U          ]'} );
    $this->{LDAP}->modify( $dn , delete  => 'shadowExpire' );
}
#-----------------------------------------------------------------------

=item B<archiv_user(dn)>

Archiv an user.

EXAMPLE:

 archiv_user('uid=micmou,ou=people,dc=schule,dc=de');

=cut

sub archiv_user($)
{
    my $this = shift;
    my $dn = shift;

    my $uid = get_name_of_dn($dn);

    $this->connect_imap();
    #TODO Is not ready

}
#-----------------------------------------------------------------------

=item B<delete_user(dn)>

Delets an user. Removes him from all groups, deletes his mail boxes, and sieve entries.

EXAMPLE:

 delete_user('uid=micmou,ou=people,dc=schule,dc=de');

=cut

sub delete_user($)
{
    my $this = shift;
    my $dn = shift;

    my $uid         = get_name_of_dn($dn);
    my $uidnumber   = $this->get_attribute($dn,'uidnumber');
    my $homedir     = $this->get_attribute($dn,'homedirectory');
    my $school_base = $this->get_school_base($dn);
    my $home_base   = '/home';
    if( $school_base ne $this->{LDAP_BASE} )
    {
      $home_base = $this->get_school_config('SCHOOL_HOME_BASE',$school_base);
    }  
    #First we clean up the mail system
    $this->{IMAP}->delete("user/$uid");

    #Now we delets the user from the groups.
    foreach my $group ( $this->get_groups_of_user($dn,1) )
    {
      $this->delete_user_from_group($dn,$group);
    }

    #Now we clean up sieve
    $this->connect_sieve($uid);
    $this->{SIEVE}->deleteScript('filter.sieve');

    #Now we deletes the files created by the user;
    system("/usr/share/oss/tools/oss_del_user_files $uid $uidnumber $home_base $homedir");

    #Now we delets the LDAP-entries of the user
    $this->delete_ldap_children($dn);
    $this->{LDAP}->delete($dn);

}
#-----------------------------------------------------------------------

=item B<delete_user_from_group(userDN,groupDN)>

Delets an user from a group. Delets the acls of the on the shared mailbox too.

EXAMPLE:

 delete_user_from_group('uid=micmou,ou=people,dc=schule,dc=de','cn=5A,ougroup,dc=schule,dc=de');

=cut

sub delete_user_from_group($$)
{
    my $this = shift;
    my $udn  = shift;
    my $gdn  = shift;

    my $gidnumber   = $this->get_attribute($gdn,'gidnumber');
    my $uid         = get_name_of_dn($udn);

    $this->{IMAP}->deleteacl(get_name_of_dn($gdn),$uid);
    $this->{LDAP}->modify($gdn, delete=> { member    => $udn }); 
    $this->{LDAP}->modify($gdn, delete=> { memberUID => $uid }); 
    $this->{LDAP}->modify($udn, delete=> { OXGroupID => $gidnumber });
}
#-----------------------------------------------------------------------

=item B<get_classes_of_user(dn)>

Get the classes of the user.

EXAMPLE:

  my  

=cut

sub get_classes_of_user($)
{
    my $this   = shift;
    my $dn     = shift;
    my @groups = ();

    my $base   = $this->get_school_base($dn);
    my $mesg   = $this->{LDAP}->search( base   => 'ou=group,'.$base,
			                filter => "(&(grouptype=class)(member=$dn))",
			                scope  => 'one',
			                attrs  => [ 'dn' ]
    );
    if( $mesg->code() )
    {
       return undef;
    }
    foreach my $entry ( $mesg->entries() )
    {
      push @groups, $entry->dn();
    }
    return @groups;
}
#-----------------------------------------------------------------------


=item B<get_groups_of_user(dn,all)>

Get the groups the user. If all is true all groups will be searched not
only the schoolGroups.

=cut

sub get_groups_of_user
{
    my $this   = shift;
    my $dn     = shift;
    my $all    = shift || 0;
    my @groups = ();
    my $primarygroup = $this->get_primary_group_of_user($dn);
    my $filter =  "(&(member=$dn)(objectclass=schoolGroup))";

    if( $all )
    {
      $filter =  "(member=$dn)";
    }

    my $mesg = $this->{LDAP}->search( base   => $this->{LDAP_BASE},
			              filter => $filter,
#TODO it is only prov because of compability:
#				      filter => "(&(member=$dn)(|(objectclass=schoolGroup)(objectclass=SchoolClass)(objectclass=SchoolPrimaryGroup)(objectclass=schoolWorkGroup))",
			              scope  => 'sub',
			              attrs  => [ 'dn' ]
    );
    if( $mesg->code() )
    {
       return undef;
    }
    foreach my $entry ( $mesg->entries() )
    {
      if( $all || ($entry->dn() ne $primarygroup) )
      {
        push @groups, $entry->dn();
      }
    }
    return @groups;
}
#-----------------------------------------------------------------------

=item B<get_primary_group_of_user(dn)>

Get the primary group of an user.

=cut

sub get_primary_group_of_user($)
{
    my $this    = shift;
    my $dn      = shift;

    my $gidnumber = $this->get_attribute($dn,'gidnumber');
    my $base      = $this->get_school_base($dn);
    my $mesg = $this->{LDAP}->search( base   => 'ou=group,'.$base,
			              filter => "gidnumber=$gidnumber",
			              scope  => 'one',
			              attrs  => [ 'dn' ]
    );
    if( $mesg->code() ||  !$mesg->count )
    {
       return undef;
    }
    return $mesg->entry(0)->dn();
}
#-----------------------------------------------------------------------

=item B<get_fquota(dn)>

Returns the file system quota of an user.

EXAMPLE:

  my ( $quotalimit, $quotavalue ) = get_fquota('uid=varkpete,ou=people,dc=extis,dc=de');

=cut

sub get_fquota($)
{
    my $this = shift;
    my $dn = shift;
    my $uid = get_name_of_dn($dn);

    my $quota = `/usr/sbin/get_quota -l $uid | gawk \'{ if(/\\/dev\\//)  print \$3 " " \$2 }\'`;
    chomp $quota;
    my ($q_val,$q_used) = split / /,$quota;
    if( defined $q_val )
    {
      $q_val = $q_val / 1024;
    }
    else
    {
      $q_val = '';
    }
    if( defined $q_used )
    {
      $q_used = $q_used / 1024;
    }
    else
    {
      $q_used = '';
    }
    return ($q_val,$q_used);
}
#-----------------------------------------------------------------------

=item B<get_quota(dn)>

Returns the mailsystem quota of an user

EXAMPLE:

  my ( $quotalimit, $quotavalue ) = get_quota('uid=varkpete,ou=people,dc=extis,dc=de');

=cut

sub get_quota($)
{
    my $this = shift;
    my $dn = shift;

    my ($q_val,$q_used);

    my $uid = get_name_of_dn($dn);

    my $anf = sub
    {
        my $self = shift;
        my $resp = shift;
        
        $q_val  = $resp->limit("STORAGE") / 1024 || '';
        $q_used = $resp->usage("STORAGE") / 1024 || '';
    };

    $this->{IMAP}->set_untagged_callback('quota', $anf);

    my $resp = $this->{IMAP}->getquotaroot("user/$uid");
   
    return ($q_val,$q_used);
}
#-----------------------------------------------------------------------

=item B<get_primary_group(dn)>

Returns the dn of a the template user of the user given by the dn. 

EXAMPLE:

  my $dn = get_primary_group(role,schoolbase);

=cut

sub get_primary_group
{
    my $this = shift;
    my $role = shift;
    my $base = shift || $this->{LDAP_BASE};
    
    my $mesg = $this->{LDAP}->search( base   => 'ou=group,'.$base,
                                      scope  => 'one',
          		              filter => "(role=$role)",
          		            );
    
    if( $mesg->code() || $mesg->count() != 1 )
    {
      return undef;
    }
    return $mesg->entry(0)->dn();
   
}

=item B<get_template(dn)>

Returns the dn of a the template user of the user given by the dn. 

EXAMPLE:

  my $template = get_template($dn);

=cut

sub get_template($)
{
    my $this = shift;
    my $dn	= shift;
  
    my $gidnumber = $this->get_attribute($dn,'gidnumber');

    my $mesg = $this->{LDAP}->search( base   => $this->{LDAP_BASE},
                                      scope  => 'sub',
				      filter => "(&(role=template)(gidnumber=$gidnumber))",
				);

    if( $mesg->code() || $mesg->count() != 1 )
    {
      return undef;
    }
    return $mesg->entry(0)->dn();
   
}
#-----------------------------------------------------------------------

=item B<get_user(dn,[list of attributes])>

Returns the datas of an user.

EXAMPLE:

  my $user = get_user('uid=varkpete,ou=people,dc=extis,dc=de');
  my $user = get_user('uid=varkpete,ou=people,dc=extis,dc=de',[ 'birthday', 'preferreddrink'] );

=cut

sub get_user
{
    my $this         = shift;
    my $dn           = shift;
    my $attrs        = shift || $this->{DEFAULT_USER_ATTRIBUTES};

    my $admin        = 0;
    my $mesg = $this->{LDAP}->search( base   => $dn,
                          scope  => 'base',
                          filter => '(objectclass=SchoolAccount)',
                          attrs  => $attrs
                        );
    if( ! $mesg )
    {
        $this->{ERROR}->{text} = "ERROR Could not search the user";
	return undef;
    }
    my $result = $mesg->as_struct;
    #Getting the mail quota
    if( contains('quota',$attrs))
    {
      my($q_val,$q_used) = $this->get_quota($dn);
      push @{$result->{$dn}->{quota}},"$q_val";
      push @{$result->{$dn}->{quotaused}},"$q_used";
    }

    #Getting the filesystem quota
    if( contains('fquota',$attrs))
    {
      my($q_val,$q_used) = $this->get_fquota($dn);
      push @{$result->{$dn}->{fquota}},"$q_val";
      push @{$result->{$dn}->{fquotaused}},"$q_used";
    }  

    #Getting the groups
    if( contains('group',$attrs))
    {
      my $primary_group = $this->get_primary_group_of_user($dn);
      push @{$result->{$dn}->{primarygroup}},$primary_group;
      foreach my $grp ($this->get_groups_of_user($dn))
      {
        push @{$result->{$dn}->{group}},$grp;
        if( $grp =~ /^cn=sysadmins,/ || $grp =~ /^cn=.*-sysadmins,/ )
        {
          $admin = 1;
        }
      }
      if( $admin || $primary_group =~ /^cn=sysadmins,/  || $primary_group =~ /^cn=.*-sysadmins,/ )
      {
        push @{$result->{$dn}->{admin}},'yes';
      }
      else
      {
        push @{$result->{$dn}->{admin}},'no';
      }
    }   
    return $result;
}

#-----------------------------------------------------------------------

=item B<get_users_of_group(dn,hash)>

Returns the member of a group. If C<hash> is true, the result is a hash else
it is an array of the members dns.

EXAMPLE:

hash:

  my $users = $oss->get_users_of_group('cn=10A,ou=group,dc=extis,dc=de',1);
  foreach $user ( keys %{$users} ) {
    print $users->{$user}->{cn}." :: ".$users->{$user}->{cn}." :: ".$users->{$user}->{description}."\n";
  }

array:

  my @users = $oss->get_users_of_group('cn=10A,ou=group,dc=extis,dc=de',0);
  foreach $user ( @users ) {
     $oss->disable_user($user}; 
  }

=cut

sub get_users_of_group
{
    my $this        = shift;
    my $dn          = shift;
    my $hash        = shift || 0;

    my $values      = $this->get_attributes($dn,['member']);
    my @members     = @{$values->{member}};
    my $result      = {};
    if( $hash )
    {
      foreach my $member ( @members )
      {
        push @{$result->{$member}->{uid}}, $this->get_attribute($member,'uid'); 
        push @{$result->{$member}->{cn}}, $this->get_attribute($member,'cn'); 
        push @{$result->{$member}->{description}}, $this->get_attribute($member,'description'); 
      }
      return $result;
    }
    return @members;
}

#-----------------------------------------------------------------------

=item B<get_students_of_group(dn)>

Returns the students of a group. If C<hash> is true, the result is a hash else
it is an array of the members dns.

EXAMPLE:

hash:

  my $students = $oss->get_students_of_group('cn=10A,ou=group,dc=extis,dc=de',1);
  foreach $user ( keys %{$students} ) {
    print $students->{$user}->{cn}." :: ".$students->{$user}->{cn}." :: ".$students->{$user}->{description}."\n";
  }

array:

  my @students = $oss->get_students_of_group('cn=10A,ou=group,dc=extis,dc=de',0);
  foreach $user ( @students ) {
     $oss->disable_user($user}; 
  }

=cut

sub get_students_of_group($)
{
    my $this        = shift;
    my $dn          = shift;
    my $hash        = shift || 0;
    my $values      = $this->get_attributes($dn,['member']);
    my @members     = @{$values->{member}};
    my $result      = {};

    if( $hash )
    {
      foreach my $member ( @members )
      {
        if( $this->is_student($member) )
	{
          push @{$result->{$member}->{uid}}, $this->get_attribute($member,'uid'); 
          push @{$result->{$member}->{cn}}, $this->get_attribute($member,'cn'); 
          push @{$result->{$member}->{description}}, $this->get_attribute($member,'description'); 
        }
      }
      return $result;
    }
    return @members;
}

#-----------------------------------------------------------------------

=item B<get_teachers_of_group(dn)>

Returns the teachers of a group. If C<hash> is true, the result is a hash else
it is an array of the members dns.

EXAMPLE:

hash:

  my $teachers = $oss->get_teachers_of_group('cn=10A,ou=group,dc=extis,dc=de',1);
  foreach $user ( keys %{$teachers} ) {
    print $teachers->{$user}->{cn}." :: ".$teachers->{$user}->{cn}." :: ".$teachers->{$user}->{description}."\n";
  }

array:

  my $teachers = $oss->get_teachers_of_group('cn=10A,ou=group,dc=extis,dc=de',0);
  foreach @user ( @teachers ) {
     $oss->disable_user($user}; 
  }

=cut

sub get_teachers_of_group($)
{
    my $this        = shift;
    my $dn          = shift;
    my $hash        = shift || 0;
    my $values      = $this->get_attributes($dn,['member']);
    my @members     = @{$values->{member}};
    my $result      = {};

    if( $hash )
    {
      foreach my $member ( @members )
      {
        if( $this->is_teacher($member) )
	{
          push @{$result->{$member}->{uid}}, $this->get_attribute($member,'uid'); 
          push @{$result->{$member}->{cn}},  $this->get_attribute($member,'cn'); 
          push @{$result->{$member}->{description}}, $this->get_attribute($member,'description'); 
        }
      }
      return $result;
    }

    return @members;
}

#-----------------------------------------------------------------------

=item B<is_user(dn)>

Returns 1 if the dn is a dn of an user.

=cut

sub is_user($)
{
    my $this   = shift;
    my $dn     = shift;

    my $result = $this->{LDAP}->search(
    			base	=> $dn,
			scope	=> 'base',
			filter	=> 'objectclass=posixAccount',
			attrs   => ['dn']
    );
    if( defined $result && $result->count ==1 )
    {
      return 1;
    }
    return 0;
}
#-----------------------------------------------------------------------

=item B<is_teacher(dn)>

Returns 1 if the dn is a dn of a teacher.

=cut

sub is_teacher($)
{
    my $this   = shift;
    my $dn     = shift;

    # TODO
    return 0;
}
#-----------------------------------------------------------------------

=item B<is_student(dn)>

Returns 1 if the dn is a dn of a student.

=cut

sub is_student($)
{
    my $this   = shift;
    my $dn     = shift;
    # TODO
    return 0;
}
#-----------------------------------------------------------------------

=item B<login(dn,password,remote-IP,session)>

Check the login parameter of the user, and returns his attributes.

C<dn> is the dn of the user how want to login.

C<password> is the user's password.

C<remote-IP> ist the IP-Address of the machine the login request comming from.

C<session> shows if a session must be created or not (1|0).

=cut

sub login
{
    my $this = shift;
    my $dn       = shift;
    my $password = shift;
    my $remote   = shift;
    my $session  = shift;
    my $uid      = '';
    my $lang     = 'EN';
    my $mesg;
    # TODO THESE VARIABLES MUST BE CONFIGURABLE
    my $timeout = 3600;
    # 1 = unix / 2 = ssl / 3 = plain
    my $connection_mode = 3;


    #Now we search the account
    $mesg = $this->{LDAP}->search( base   => $dn,
    			   	   scope  => 'base',
			           attr   => [],
			           filter => "(!(loginDisabled=yes))"
			         );
    if( $mesg->code() || $mesg->count() != 1 )
    {
      $this->{ERROR}->{text} = "Login denied\n";
      return undef;
    }
    $mesg = $this->{LDAP}->bind( $dn, password =>  $password);
    if( $mesg->code != 0)
    {
      $this->{ERROR}->{text} = "Login failed\n";
      return undef;
    }
   
    #Now we get all the data of the user
    $mesg = $this->{LDAP}->search( base   => $dn,
    			           scope  => 'base',
			           filter => 'objectclass=*',
			         );
    my $result = $mesg->as_struct;
    my %data   = ();
    foreach my $attr ( keys %{$result->{$dn}} )
    {
      $data{$attr} = join "::",@{$result->{$dn}->{$attr}};
    }
    if( defined $data{preferredlanguage} ) {
	$lang = $data{preferredlanguage};
    }
    $uid = $data{uid};

    #-- calculate if the account is expired
    my $days_since_1970 = int(timelocal(localtime()) / 3600 / 24);

    if( defined $data{shadowExpire} && $data{shadowExpire} ne "" && $data{shadowExpire} < $days_since_1970)
    {
      $this->{ERROR}->{text} = "Login expired";
      return undef;
    }

    # Create a new session if neccessary
    if( $session )
    {
      my $dref = \%data;
      my $rand = rand((time)*$$);
      my $ID = md5_hex($rand.$dref);

      my $authdata = encode_base64($uid."\1".$password."\1".$lang."\1".$remote);
      $authdata =~ s/\n//g;

      my $SOCK = getSocket($connection_mode);
      my $timestamp = timelocal(localtime());
      my $frozen = freeze($dref);
      $frozen = unpack('H*',$frozen);

      print $SOCK "add: $timestamp $timeout $ID $authdata $frozen\0";
      $SOCK->flush();
      push @{$result->{$dn}->{'SESSIONID'}}, $ID;
   }

   #Now we search  for the users group
   push @{$result->{$dn}->{'group'}},$this->get_groups_of_user($dn);
   push @{$result->{$dn}->{'primarygroup'}},$this->get_primary_group_of_user($dn);
   return $result; 
}
#-----------------------------------------------------------------------

=item B<set_password(dn, password, mustchange, sso, hash)>

Set the user password (samba and LDAP).

 
EXAMPLE:

  set_password('uid=varkpete,ou=people,dc=extis,dc=de','12345678',1,0,'smd5');

=cut

sub set_password($$$$$)
{
    my $this = shift;
    my $dn         = shift;
    my $password   = shift;
    my $mustchange = shift;
    my $sso        = shift;
    my $pwmech     = shift;
    
    my $uid = get_name_of_dn($dn);
    my $crypt_password = hash_password($pwmech,$password);
    my $days_since_1970 = int(timelocal(localtime()) / 3600 / 24);
    my ( $lm, $nt ) = ntlmgen $password;
    my @mod_op;
    my @mod_op_1;
    my @mod_op_2;
    push @mod_op, "replace", [ "sambaLMPassword", "$lm" ];
    push @mod_op, "replace", [ "sambaNTPassword", "$nt" ];
    push @mod_op, "replace", [ "userpassword", "$crypt_password" ];
    if( $mustchange )
    {
        push @mod_op, "replace", [ "shadowlastchange", "0" ];
        push @mod_op, "replace", [ "sambaPwdMustChange", "0" ];
    }
    else
    {
        push @mod_op, "replace", [ "shadowlastchange", "$days_since_1970" ];
        push @mod_op_1, "delete",  [ "sambaPwdMustChange" , []];
    }
    if( $sso )
    {
    	my $authdata =  encode_base64($uid."\1".$password);
    	push @mod_op, "replace", [ "authData", "$authdata" ];
    }
    else
    {
    	push @mod_op_2, "delete", ["authData" , []];
    }
    
    $this->{LDAP}->modify( $dn, changes => \@mod_op );
    if( @mod_op_1 )
    {
      $this->{LDAP}->modify( $dn, changes => \@mod_op_1 );
    }
    if( @mod_op_2 )
    {
      $this->{LDAP}->modify( $dn, changes => \@mod_op_2 );
    }

}
#-----------------------------------------------------------------------

=item B<set_fquota(dn,MB)>

Sets the file system quota of an user.

EXAMPLE:

  $oss->set_fquota('uid=varkpete,ou=people,dc=extis,dc=de',200);

=cut

sub set_fquota
{
    my $this    = shift;
    my $dn      = shift;
    my $fquota  = shift;
    my $fsystem = shift || '-a';
    
    my $uidnumber = $this->get_attribute($dn,'uidnumber');
    $fquota      *= 1024;

    system("/usr/sbin/setquota -u $uidnumber $fquota $fquota 0 0 $fsystem");
}
#-----------------------------------------------------------------------

=item B<set_quota(dn,MB)>

Sets the mail system quota of an user.

EXAMPLE:

  $oss->set_quota('uid=varkpete,ou=people,dc=extis,dc=de',200);

=cut

sub set_quota
{
    my $this    = shift;
    my $dn      = shift;
    my $quota   = shift;
    my $uid     = get_name_of_dn($dn);

    my @qarray = ();
    if( $quota )
    {
      @qarray = ("STORAGE", $quota * 1024 );
    }
    $this->{IMAP}->setquota("user/$uid", @qarray);
}
#-----------------------------------------------------------------------

#######################################################################################
# Utilities to manipulate the groups						      #
#######################################################################################

=item B<delete_group(dn)>

Delets a group. Removes his mail box group directory and LDAP-Entries.

EXAMPLE:

 delete_group('cn=1008-teachers,ou=group,uniqueIdentifier=1008,dc=realschule-bayern,dc=info');

=cut

sub delete_group($)
{
    my $this = shift;
    my $dn = shift;

    my $cn          = get_name_of_dn($dn);
    my $gidnumber   = $this->get_attribute($dn,'gidnumber');
    my $grouptype   = $this->get_attribute($dn,'grouptype');
    my $school_base = $this->get_school_base($dn);
    my $values      = $this->get_attributes($dn,['member']);
    my @member      = ();
    if( defined $values)
    {
	@member = @{$values->{member}};
    }
    my $prefix      = $this->get_school_config('SCHOOL_GROUP_PREFIX',$dn) || '';
    my $homebase    = $this->get_school_config('SCHOOL_HOME_BASE',$dn);
    my $cnhome      = $cn;

    if( $prefix ne '' )
    {
      $cnhome =~ s/^$prefix//;
    }

    if( $cnhome ne '' && $homebase ne '' )
    {
      system("rm -r $homebase/groups/$cnhome");
    }  

    if( $grouptype eq 'primarygroup' && $#member > 1 )
    {
      $this->{ERROR}->{text} = "ERROR Can not delete Primary group if it contains members\n";
      return undef;
    }

    $this->{IMAP}->delete($cn);

    foreach my $udn (@member)
    {
      $this->{LDAP}->modify($udn, delete => { OXGroupID => $gidnumber });
    }

    $this->delete_ldap_children($dn);
   
    $this->{LDAP}->delete($dn);

}
#-----------------------------------------------------------------------

=item B<get_gidnumber(cn)>

Get the gidnumber of a group.

=cut

sub get_gidnumber($)
{
    my $this = shift;
    my $cn      = shift;

    my $mesg = $this->{LDAP}->search( base   => $this->{LDAP_BASE},
			              filter => "(&(objectClass=posixGroup)(cn=$cn))",
			              scope  => 'sub',
			              attrs  => [ 'gidNumber' ]
                                    );
    if( $mesg->code() )
    {
       return -1;
    }
    if( $mesg->count() != 1 )
    {
       return -2;
    }
    return $mesg->entry(0)->get_value('gidNumber');
}
#-----------------------------------------------------------------------

=item B<get_fquota_group(dn)>

Returns the file system quota of a group.

EXAMPLE:

  my ( $quotalimit, $quotavalue ) = get_fquota('cn=10A,ou=group,dc=extis,dc=de');

=cut

sub get_fquota_group($)
{
    my $this = shift;
    my $dn = shift;
    my $cn = get_name_of_dn($dn);

    my $quota = `/usr/sbin/get_quota -l -g $cn | gawk \'{ if(/\\/dev\\//)  print \$3 " " \$2 }\'`;
    chomp $quota;
    my ($q_val,$q_used) = split / /,$quota;
    if( defined $q_val )
    {
      $q_val = $q_val / 1024;
    }
    else
    {
      $q_val = '';
    }
    if( defined $q_used )
    {
      $q_used = $q_used / 1024;
    }
    else
    {
      $q_used = '';
    }
    return ($q_val,$q_used);
}
#-----------------------------------------------------------------------

=item B<get_quota_group(dn)>

Returns the mailsystem quota of a group

EXAMPLE:

  my ( $quotalimit, $quotavalue ) = get_quota('cn=10A,ou=group,dc=extis,dc=de');

=cut

sub get_quota_group($)
{
    my $this = shift;
    my $dn = shift;

    my ($q_val,$q_used);

    my $cn = get_name_of_dn($dn);

    my $anf = sub
    {
        my $self = shift;
        my $resp = shift;
        
        $q_val  = $resp->limit("STORAGE") / 1024 || 0;
        $q_used = $resp->usage("STORAGE") / 1024 || 0;
    };

    $this->{IMAP}->set_untagged_callback('quota', $anf);

    my $resp = $this->{IMAP}->getquotaroot($cn);
   
    return ($q_val,$q_used);
}
#-----------------------------------------------------------------------

=item B<get_group(dn)>

Returns the datas of a group.

EXAMPLE:

  my $group = get_group('cn=10,ou=group,dc=extis,dc=de');

=cut

sub get_group
{
    my $this         = shift;
    my $dn           = shift;

    my $admin        = 0;

    my $mesg = $this->{LDAP}->search( base   => $dn,
                          scope  => 'base',
                          filter => '(objectclass=SchoolGroup)',
                          attrs  => $this->{DEFAULT_GROUP_ATTRIBUTES}
                        );
    if( ! $mesg )
    {
        $this->{ERROR}->{text} = "ERROR Could not search the group";
	return undef;
    }
    my $result = $mesg->as_struct;
    #Getting the mail quota
    my($q_val,$q_used) = $this->get_quota($dn);
    push @{$result->{$dn}->{quota}},"$q_val";
    push @{$result->{$dn}->{quotaused}},"$q_used";

    #Getting the filesystem quota
    ($q_val,$q_used) = $this->get_fquota($dn);
    push @{$result->{$dn}->{fquota}},"$q_val";
    push @{$result->{$dn}->{fquotaused}},"$q_used";

    return $result;
}
#-----------------------------------------------------------------------

=item B<is_group(dn)>

Returns 1 if the dn is a dn of a group.

=cut

sub is_group($)
{
    my $this   = shift;
    my $dn     = shift;

    my $result = $this->{LDAP}->search(
    			base	=> $dn,
			scope	=> 'base',
			filter	=> 'objectclass=posixGroup',
			attrs   => ['dn']
    );
    if( defined $result && $result->count ==1 )
    {
      return 1;
    }
    return 0;
}
#-----------------------------------------------------------------------

=item B<is_school_group(dn)>

Returns 1 if the dn is a dn of a school group.

=cut

sub is_school_group($)
{
    my $this   = shift;
    my $dn     = shift;

    my $result = $this->{LDAP}->search(
    			base	=> $dn,
			scope	=> 'base',
			filter	=> 'objectclass=schoolGroup',
			attrs   => ['dn']
    );
    if( defined $result && $result->count ==1 ) {
      return 1;
    }
    return 0;
}
#-----------------------------------------------------------------------

=item B<set_fquota_group(dn,MB)>

Sets the file system quota of a group.

EXAMPLE:

  $oss->set_fquota('cn=10A,ou=group,dc=extis,dc=de',200);

=cut

sub set_fquota_group
{
    my $this    = shift;
    my $dn      = shift;
    my $fquota  = shift;
    my $fsystem = shift || '-a';
    
    my $gidnumber = $this->get_attribute($dn,'gidnumber');
    $fquota      *= 1024;

    system("/usr/sbin/setquota -g $gidnumber $fquota $fquota 0 0 $fsystem");
}
#-----------------------------------------------------------------------

=item B<set_quota_group(dn,MB)>

Sets the mail system quota of an user.

EXAMPLE:

  $oss->set_quota_group('cn=10A,ou=group,dc=extis,dc=de',200);

=cut

sub set_quota_group
{
    my $this    = shift;
    my $dn      = shift;
    my $quota   = shift;
    my $cn      = get_name_of_dn($dn);

    my @qarray = ();
    if( $quota )
    {
      @qarray = ("STORAGE", $quota * 1024 );
    }
    $this->{IMAP}->setquota($cn, @qarray);
}
#-----------------------------------------------------------------------

#######################################################################################
# Utilities to manipulate the schools						      #
#######################################################################################

=item B<delete_school(dn)>

Delets a school. Removes all users and groups and LDAP-entries.

EXAMPLE:

 delete_school('uniqueIdentifier=1008,dc=realschule-bayern,dc=info');

=cut

sub delete_school($)
{
    my $this = shift;
    my $dn = shift;

    if( $dn eq $this->{LDAP_BASE} )
    {
	$this->{ERROR}->{text} = "ERROR can not delete the base institute.";
	return 0;
    }
    my $homebase    = $this->get_school_config('SCHOOL_HOME_BASE',$dn);
    my $domain      = $this->get_school_config('SCHOOL_DOMAIN',$dn);

    foreach my $udn ( $this->get_school_users('*',$dn) )
    {
      my $uid = get_name_of_dn($udn);
      $this->{IMAP}->delete("user/$uid");
    }
    foreach my $gdn ( $this->get_school_groups('*',$dn) )
    {
      my $cn = get_name_of_dn($gdn);
      $this->{IMAP}->delete("$cn");
    }

    if( $homebase && $homebase =~ /^\/home/ )
    {
      system("rm -r $homebase");
    }
    if( $domain && -e "/etc/apache2/vhosts.d/$domain.conf" )
    {
	system("rm /etc/apache2/vhosts.d/$domain.conf");
    }

    #Now we delets the LDAP-entries of the user
    $this->delete_ldap_children($dn);
    $this->{LDAP}->delete($dn);

    #Delete the dns-Entries
    my $mesg = $this->{LDAP}->search(
    		base   => $this->{SYSCONFIG}->{DNS_BASE},
		scope  => 'sub',
		filter => "(|(zoneName=$domain)(pTRRecord=*$domain.))"
    );
    foreach my $entry ($mesg->all_entries)
    {
      $this->{LDAP}->delete($entry->dn);
    }  

    return 1;

}
#-----------------------------------------------------------------------

=item B<get_school_config>

Returns an configurations value of a school

EXAMPLE:

 my $domain = get_school_config('SCHOOL_DOMAIN',DN-of-the-school);

=cut

sub get_school_config
{
    my $this = shift;
    my $key  = shift;
    my $base = shift || $this->{LDAP_BASE};
    my $mesg;

    $mesg = $this->{LDAP}->search( base   => "ou=sysconfig,$base",
                                   scope  => 'one',
                                   filter => "configurationKey=$key",
                                 );
    if( $mesg && $mesg->count() == 1)
    {
      return $mesg->entry(0)->get_value('configurationValue');
    }
    return undef;
}
#-----------------------------------------------------------------------

=item B<get_school(school_dn)>

Delivers the settings of a school.

   my $school   = get_school('uniqueIdentifier=1008,dc=realschule-bayern,dc=info');

=cut

sub get_school
{
    my $this        = shift;
    my $dn          = shift;

    my $mesg = $this->{LDAP}->search( base   => $dn,
                                      scope  => 'base',
                                      filter => '(objectclass=School)'
                        );

    my $result = $mesg->as_struct;
    $this->{LDAP_BASE} = $dn;
    $this->init_sysconfig();

    foreach my $key ( keys %{$this->{SYSCONFIG}} )
    {
      push @{$result->{$dn}->{$key}}, $this->{SYSCONFIG}->{$key};
    }
    return $result;
}
#-----------------------------------------------------------------------

=item B<get_schools()>

Delivers the DNs of all schools in the database.

   my @school   = $oss->get_schools();

=cut

sub get_schools
{
    my $this        = shift;
    my @dns         = ();

    my $mesg = $this->{LDAP}->search( base   => $this->{LDAP_BASE},
                                      scope  => 'one',
                                      filter => '(objectclass=School)',
				      attrs  => ['dn']
                        );

    foreach my $entry ( $mesg->entries )
    {
      push @dns, $entry->dn();
    }
    return @dns;
}
#-----------------------------------------------------------------------

=item B<get_school_base(oid|dn_of_an_object_in_the_school)>

Returns the base dn of a school. Oid can be the oid of a school or a dn
of an object in the school.

EXAMPLE:

  my $base = get_school_base('12345');

  my $base = get_school_base('uid=varkpete,ou=people,dc=extis,dc=de');
  
  my $base = get_school_base('uid=varkpete,ou=people,uniqueidentifier=12345,dc=extis,dc=de');
  
=cut

sub get_school_base($)
{
    my $this = shift;
    my $oid  = shift;

    if( $oid =~ /$this->{LDAP_BASE}$/i )
    {
      #This  is a DN not an oid
      if( $oid =~ /(uniqueidentifier=.*)/i )
      {
        return $1;
      }
      return $this->{LDAP_BASE};
    }

    my  $mesg = $this->{LDAP}->search( base   => $this->{LDAP_BASE},
                             scope  => 'one',
                             filter => "(&(uniqueidentifier=$oid)(objectclass=School))"
                    );
    if( $mesg && $mesg->count() == 1)
    {
      return $mesg->entry(0)->dn();
    }
    $this->{ERROR}->{text} = "EROOR Can not find a school for the identifier $oid";
    return undef;
}
#-----------------------------------------------------------------------

=item B<get_school_groups(group_type,school_dn)>

Delivers the groups of a school.

   my @all_groups    = get_school_groups('*');
   my @primarygroups = get_school_groups('primary');

=cut

sub get_school_groups
{
    my $this        = shift;
    my $type        = shift;
    my $school_base = shift || $this->{LDAP_BASE};
    $school_base    = $this->get_school_base($school_base);
    my @dn          = ();

    my $filter      = '(objectclass=*)';

    if( $type ne '*' )
    {
      $filter      = "(&(objectclass=schoolGroup)(grouptype=$type)";
    }

    my $mesg = $this->{LDAP}->search( base   => 'ou=group,'.$school_base,
    				      filter => $filter,
				      scope  => 'one',
				      attrs  => [ 'dn' ]
				    );  
    foreach my $entry ( $mesg->entries() )
    {
      push @dn, $entry->dn();
    }
    return @dn;
}

#-----------------------------------------------------------------------

=item B<get_school_users(role,school_dn)>

Delivers the users of a school.

   my @all_user = get_school_users('*');
   my @teachers = get_school_users('teacher');


=cut

sub get_school_users {
    my $this        = shift;
    my $role        = shift;
    my $school_base = shift || $this->{LDAP_BASE};
    my @dn          = ();
    my $filter      = '(objectclass=schoolAccount)';

    if( $role ne '*' )
    {
      $filter      = "(&(objectclass=schoolAccount)(role=$role)";
    }

    my $mesg = $this->{LDAP}->search( base   => 'ou=people,'.$school_base,
    				      filter => $filter,
				      scope  => 'one',
				      attrs  => [ 'dn' ]
				    );  
    foreach my $entry ( $mesg->entries() )
    {
      push @dn, $entry->dn();
    }
    return @dn;
}

#-----------------------------------------------------------------------

#######################################################################################
# Utilities to manipulate the workstations / images				      #
#######################################################################################

=item B<get_image>

Returns the DN of image object to a software and hardware configuration 

EXAMPLE:

 my $VALUE = get_image('hwconf1','swconf2');

=cut

sub get_image($$)
{
    my $this = shift;
    my $hw   = shift;
    my $sw   = shift;

    my $mesg = $this->{LDAP}->search( base   => $this->{SYSCONFIG}->{COMPUTERS_BASE},
                                      scope  => 'one',
                                      filter => "(&(configurationValue=HW=$hw)(configurationValue=SW=$sw)(configurationValue=TYPE=IMAGE))",
                                      attrs  => [ 'dn' ]
                            );
    if( $mesg && $mesg->count() == 1)
    {
        return $mesg->entry(0)->dn();
    }
    return undef;
}

#-----------------------------------------------------------------------

=item B<add_host_dns_zone(hostname,IP-Address,zone)>

Insert a new host into the DNS server

EXAMPLE:

 add_host_dns_zone("www","192.168.0.2","168.192.IN-ADDR.ARPA");
 add_host_dns_zone("www","192.168.0.2","schule.de");

=cut

sub add_host_dns_zone
{
    my $this     = shift;
    my $hostname = shift;
    my $ip       = shift;
    my $zone     = shift;

    my $mesg = $this->{LDAP}->search( base    => $this->{LDAP_BASE},
       				      scope   => 'sub',
				      attrs   => ['dn'],
				      filter  => "(&(relativeDomainName=@)(zoneName=$zone))"
			);
    if( $mesg->code)
    {
         $this->ldap_error($mesg);
         return undef;
    }
    my $base = $mesg->entry(0)->dn;

    if( $zone =~ /((\d+\.)+)IN-ADDR\.ARPA/i )
    { #Adding revers lookup entry
       my @rev_net           = split /\./, $1;
       my @full_host_address = reverse(split( /\./, $ip));
       my @host_address      = ();
       for( my $i=0; $i < 3 - $#rev_net; $i++)
       {
         $host_address[$i] = $full_host_address[$i];
       }
       my $rev_relDN   = join (".", @host_address );
       my $reverse_dn  = 'relativeDomainName='.$rev_relDN.','.$base;
       $mesg = $this->{LDAP}->search(     base    => $reverse_dn,
                                  scope   => 'base',
                                  attrs   => ['pTRRecord'],
                                  filter  => '(objectClass=*)'
                             );
       if( $mesg->code)
       {
	    $this->ldap_error($mesg);
	    return undef;
       }
       elsif( $mesg->count == 0 )
       {  # The entry does not exist yet a new on has to be created
          my $reverse_entry = Net::LDAP::Entry->new();
          $reverse_entry->dn($reverse_dn);
          $reverse_entry->add(    relativeDomainName      => $rev_relDN,
                                  pTRRecord               => "$hostname.",
                                  dNSTTL                  => '172800',
                                  zoneName                => $zone,
                                  objectClass             => ['top', 'dNSZone']
                      );
          $mesg = $this->{LDAP}->add($reverse_entry);
          if($mesg->code)
	  {
	    $this->ldap_error($mesg);
	    return undef;
          }
       }
       else
       {  # The entry is already defined just a new value to pTRRecord
          $mesg = $this->{LDAP}->modify( $reverse_dn, "add" => { "pTRRecord" => "$hostname."});
          if($mesg->code)
          {
	    $this->ldap_error($mesg);
	    return undef;
          }
       }
    }
    else
    {
       my $base= 'relativeDomainName='.$hostname.','.$base;
       $mesg = $this->{LDAP}->search(     base    => $base,
                                          scope   => 'base',
                                          attrs   => ['aRecord'],
                                          filter  => '(objectClass=*)'
                                     );
       if( $mesg->code)
       {
	    $this->ldap_error($mesg);
	    return undef;
       }
       elsif( $mesg->count == 0 )
       {  # The entry does not exist yet a new on has to be created
          my $hostentry = Net::LDAP::Entry->new();
          $hostentry->dn( $base );
          $hostentry->add(aRecord                 => $ip,
                          relativeDomainName      => $hostname,
                          dNSTTL                  => '172800',
                          zoneName                => $zone,
                          objectClass             => ['top', 'dNSZone']
                     );
         $mesg = $this->{LDAP}->add($hostentry);
         if($mesg->code)
         {
	    $this->ldap_error($mesg);
	    return undef;
         }
       }
       else
       {
          $mesg = $this->{LDAP}->modify( $base, "add" => { aRecord => $ip});
          if($mesg->code)
          {
	    $this->ldap_error($mesg);
	    return undef;
          }
       }
   }
   $this->update_soa($zone);
   return 1;
}
#-----------------------------------------------------------------------

=item B<add_host_dns(FQHN,IP-Address)>

Insert a new host into the DNS server

EXAMPLE:

 add_host_dns("www.schule.de","192.168.0.2");

=cut

sub add_host_dns($$)
{
    my $this   = shift;
    my $fqhn   = shift;
    my $ip     = shift;

    my ($hostname,$domainname) = split /\./, $fqhn, 2;
    $this->add_host_dns_zone($hostname,$ip,$domainname);

    # Now we try to create the revers lookup
    my $mesg = $this->{LDAP}->search (   base     => $this->{LDAP_BASE},
                                      filter   => '(&(relativeDomainName=@)(zoneName=*IN-ADDR.ARPA))',
				      attrs    => ['zoneName','dn']
    );
    my @lip = split /\./, $ip;
    foreach my $entry ($mesg->entries)
    {
      my $myzone = 1;
      my $zone   = $entry->get_value('zoneName');
      $zone =~ /^((\d+\.)+)IN-ADDR\.ARPA/i;

      if(defined $1)
      {
        my @ip_parts = reverse(split /\./, $1);
        for( my $i=0; $i < $#ip_parts; $i++ )
	{
	   if( $ip_parts[$i] ne $lip[$i] ){
	     $myzone = 0;
	     last;
	   }
	}
      }
      else
      {
        $this->{ERROR}->{text} = "ERROR Bad IN-ADDR.ARPA zone name: $zone";
	return 0;
      }
      if($myzone)
      {
	$this->add_host_dns_zone($hostname,$ip,$zone);
	last;
      }
    }
    
    return 1;
}
=item B<add_workstation()>

EXAMPLE:

=cut

sub add_workstation {
    my $this = shift;
    my $ip   = shift;
    my $room = shift;
#TODO
}
#-----------------------------------------------------------------------

=item B<delete_workstation(dn)>

Delets a workstation. The workstation will be given by the dn in the 
DHCP configuration.

EXAMPLE:

=cut

sub delete_workstation 
{
    my $this = shift;
    my $dn   = shift;
    my $name = get_name_of_dn($dn);
    my $base = 'ou=DNS,'.$this->get_school_base($dn);

    my $result = $this->{LDAP}->search(  base    => $base,
                                         filter  => "dhcpHostDN=$dn",
                                         attrs   => ['dn']
                           );
    # Now we delete the entry from the corresponding DNS-forward entry
    foreach my $entry ($result->entries)
    {
        $this->{LDAP}->delete($entry->dn());
    }

    $result = $this->{LDAP}->search(  base    => $base,
                                      filter  => "pTRRecord=$name.*",
                                      attrs   => ['dn']
                           );
    # Now we delete the entry from the corresponding DNS-forward entry
    foreach my $entry ($result->entries)
    {
        $this->{LDAP}->delete($entry->dn());
    }
    # Now we delete the DHCP-entry
    $this->{LDAP}->delete($dn);

}
#-----------------------------------------------------------------------

=item B<get_workstations_of_room(dn,hash)>

Lists the workstations of a room; If C<hash> is false only an array of the
dns of the workstations will be reported.

EXAMPLE:

=cut

sub get_workstations_of_room
{
    my $this = shift;
    my $dn   = shift;
    my $hash = shift || 0;
    my $result = $this->{LDAP}->search(  base    => $dn,
					 scope   => 'one',
                                         filter  => 'objectclass=dhcpHost'
                           );
    my %ws = %{$result->as_struct};

    if( $hash )
    {
      return \%ws;
    }
    my @dns = ();
    foreach my $dn ( keys %ws )
    {
      push @dns, $dn;
    }
    return @dns;
}
#-----------------------------------------------------------------------

=item B<get_rooms() >

Returns the rooms.

EXAMPLE:

  my $rooms = $oss->get_rooms();

=cut

sub get_rooms($)
{
    my $this   = shift;

    my $result = $this->{LDAP}->search( base   => $this->{SYSCONFIG}->{DHCP_BASE},
                                       scope   => 'sub',
                                       filter  => '(&(Objectclass=SchoolRoom)(description=*))'
                              );
    return $result->as_struct;

}

=item B<get_room_of_ip(IP-address) >

Returns the the attributes of the room.

EXAMPLE:

=cut

sub get_room_of_ip
{
    my $this   = shift;
    my $ip     = shift;
    my $hash   = shift || 0;

    my $block  = new Net::Netmask($ip.'/26');
    my $roomip = $block->base();

    my $result = $this->{LDAP}->search(  base    => $this->{SYSCONFIG}->{DHCP_BASE},
					 scope   => 'sub',
					 filter  => "(&(dhcpRange=$roomip)(objectClass=schoolRoom))"
                           );
    if( $result && $result->count())
    {
      if( $hash )
      {
        return $result->as_struct;
      }
      return $result->entry(0)->dn;
    }
    return undef;
}
#-----------------------------------------------------------------------

=item B<get_workstations(dn) >

Returns the get_workstations of a room.

EXAMPLE:

  my $roomdn = get_room_of_ip($my_ip);
  my $pcs    = get_workstations($roomdn);

=cut

sub get_workstations
{
    my $this   = shift;
    my $dn     = shift;
    
    my $result = $this->{LDAP}->search(  base    => $dn,
					 scope   => 'one',
					 filter  => "(objectClass=schoolWorkstation)"
                           );
    if( $result  && $result->count() )
    {
      return $result->as_struct;
    }
    return undef;
}

=item B<get_room_access_list(dn) >

Returns the rooms service access list.

EXAMPLE:

  my $dn    = $oss->get_room_of_ip('192.168.2.23');
  my %sacls = %{$oss->get_room_access_list($dn)};

  print "Default Service Acces Status ".$sacls->{DEFAULT}."\n"; 
  foreach my $time ( sort(keys(%sacls)) ) {
    next if $time eq 'DEFAULT';
    print "Service Acces Status at $time: ".$sacls->{time}."\n"; 
  }  

=cut

sub get_room_access_list($)
{
    my $this   = shift;
    my $dn     = shift;
    my %acls   = ();

    my $result = $this->{LDAP}->search(  base    => $dn,
					 scope   => 'base',
                                         filter  => 'objectclass=schoolRoom',
					 attrs   => [ 'serviceAccesControl' ]
                           );
    if( $result && $result->count() )
    {
	my @acls  =  $result->entry(0)->get_value('serviceAccesControl');
	foreach my $acl (@acls)
	{
	  my ( $time, $value ) = split / /,$acl;
	  $acls{$time} = $value;
	}
	return \%acls;
    }
    return undef;
}
#-----------------------------------------------------------------------

=item B<get_room_name(dn) >

Returns the name (description) of the room.

EXAMPLE:

  my $dn   = $oss->get_room_of_ip('192.168.2.23');
  my $room = $oss->get_room_name($dn);

=cut

sub get_room_name($)
{
    my $this   = shift;
    my $dn     = shift;

    return $this->get_attribute($dn,'description');
}    
#-----------------------------------------------------------------------

=item B<get_room_status(dn) >

Returns the access status of the room.

EXAMPLE:

  my $dn   = $oss->get_room_of_ip('192.168.2.23');
  my $room = $oss->get_room_status($dn);

=cut

sub get_room_status($)
{
    my $this   = shift;
    my $dn     = shift;

#  TODO this subroutin is empty
}
#-----------------------------------------------------------------------

=item B<get_workstation(IP-Adress) >

Returns the access status of the room.

EXAMPLE:

  my $dn   = $oss->get_workstation('192.168.2.23');

=cut

sub get_workstation
{
    my $this   = shift;
    my $ip     = shift;
    my $hash   = shift || 0;

    my $result = $this->{LDAP}->search( base   => $this->{SYSCONFIG}->{DHCP_BASE},
                                        filter => "(&(|(dhcpHWAddress=ethernet $ip)(dhcpStatements=fixed-address $ip))(objectclass=schoolworkstation))",
                        );
    if( $result && $result->count() )
    {
      if( $hash )
      {
        return $result->as_struct;
      }
      return $result->entry(0)->dn;

    }

    return undef;

}
#-----------------------------------------------------------------------

=item B<update_soa(zone)>

Updates the soa record of a zone

EXAMPLE:

    $oss->update_soa('schule.de');
    $oss->update_soa("168.192.IN-ADDR.ARPA");

=cut

sub update_soa
{
  my $this   = shift;
  my $zone   = shift;

  # get out SOA for rev Zone and increase serial number
  my $mesg = $this->{LDAP}->search(     base   => $this->{LDAP_BASE},
                                        scope  => 'sub',
                                        filter => "(&(zoneName=$zone)(relativeDomainName=@))"
                                  );
  if( $mesg->code)
  {
    $this->ldap_error($mesg);
    return undef;
  }
  elsif( $mesg->count == 0)
  {
    $this->{ERROR} = "No such zone: $zone\n";
    return undef;
  }

  my $zone_entry = $mesg->entry(0);
  my $soa        = $zone_entry->get_value("sOARecord");
  my @soa        = split(/ /,$soa);
  my $timestamp  = $soa[2];
  my $sernr      = substr($timestamp, 8, 2);
  my $timenr     = substr($timestamp, 0, 8);
  my $timenow    = strftime("%Y%m%d",localtime);

  if( $timenr eq $timenow )
  {
    $sernr++;
  }
  else
  {
    $timenr = $timenow;
  }

  $zone_entry->replace( sOARecord =>
                  $soa[0]." ".$soa[1]." ".$timenr.$sernr." ".
                  $soa[3]." ".$soa[4]." ".$soa[5]." ".$soa[6]);

  $mesg = $zone_entry->update( $this->{LDAP} );
  if($mesg->code)
  {
    $this->ldap_error($mesg);
    return undef;
  }
  return 1;
}
#-----------------------------------------------------------------------

=item B<get_computer_config_value>

Returns a computer configurations value.

EXAMPLE:

 my $VALUE = get_computer_config_value('ProductID','swconf1');

=cut

sub get_computer_config_value($$)
{
    my $this = shift;
    my $key  = shift;
    my $conf = shift;

    return $this->get_config_value($key,'configurationKey='.$conf.','.$this->{SYSCONFIG}->{COMPUTERS_BASE});

}
#-----------------------------------------------------------------------

#######################################################################################
# Create, Read and Modify Vendor Objects					      #
#######################################################################################

=item B<create_vendor_object(dn,vendor,key,value)>

Create a new vendor object for a school object.

EXAMPLE:

    $oss->create_vendor_object( 'uid=admin,ou=people,dc=EXTIS-School,dc=de',
				'extis','rights',['read','write']
    );

=cut

sub create_vendor_object
{
  my $this   = shift;
  my $dn     = shift;
  my $vendor = shift;
  my $key    = shift;
  my $value  = shift;
  my $vbase  = 'o='.$vendor.','.$dn;
  my $kbase  = 'configurationKey='.$key.','.$vbase;
  
  if( $this->exists_dn($dn) != 1)
  {
    return undef;
  }
  my $status = $this->exists_dn($vbase);
  if( $status == 0 )
  {
    $this->{LDAP}->add( dn =>  $vbase,
    			attr => [
			      objectclass => [ 'top', 'organization' ],
		              o           => $vendor
			      ]
	       );
  }
  elsif( $status < 0 )
  {
    return undef;
  }
  $status = $this->exists_dn($kbase);
  if( $status == 0 )
  {
    $this->{LDAP}->add( dn =>  $kbase,
    			attr => [
			      objectclass        => [ 'top', 'schoolConfiguration' ],
		              configurationKey   => $key,
			      configurationValue => $value
			      ]
	       );
  }
  elsif( $status == 1 )
  {
    $this->{LDAP}->modify( $kbase, add => { configurationValue => $value } );
  }

}
#-----------------------------------------------------------------------

=item B<create_vendor_object(dn,vendor,key,value)>

Create a new vendor object for a school object.

EXAMPLE:

    $oss->get_vendor_object( 'uid=admin,ou=people,dc=EXTIS-School,dc=de',
				'extis','rights'
    );

=cut

sub get_vendor_object
{
  my $this   = shift;
  my $dn     = shift;
  my $vendor = shift;
  my $key    = shift;
  my $vbase  = 'o='.$vendor.','.$dn;
  my $kbase  = 'configurationKey='.$key.','.$vbase;

  my $mesg = $this->{LDAP}->search( base   => $kbase,
  				    scope  => 'base',
				    filter => 'objectclass=*'
  );
  return $mesg->as_struct;

}
#-----------------------------------------------------------------------
  
#######################################################################################
# Utilities which are obsolate							      #
#######################################################################################

=item B<get_group_dn(cn,dn)>

Returns the dn of a group or undef if no or more then one group was found. 

EXAMPLE:

  my $dn = get_group_dn('10A');

  my $dn = get_group_dn('10A',dn_of_the_school);

=cut

sub get_group_dn
{
    my $this         = shift;
    my $group        = shift || return undef;
    my $school_base  = shift || '';
    my $pref = '';
    my $mesg;

    my $base	= $this->{LDAP_BASE};

    # First we have to search the school prefix;
    if( $school_base ne '' )
    {
      $pref = $this->get_school_config('SCHOOL_GROUP_PREFIX',$school_base);
      if( $group !~ /^$pref/ )
      {
        $group = $pref.$group;
      }
      $base = $school_base;
    }
    
    $mesg = $this->{LDAP}->search( base   => "ou=group,$base",
    			 scope  => 'one',
          		 filter => "(&(cn=$group)(objectClass=schoolGroup))",
          		 attrs  => ['dn' ]
          		);
    if( $mesg->code() || $mesg->count() != 1 )
    {
      return undef;
    }
    return $mesg->entry(0)->dn();
}
#-----------------------------------------------------------------------
=item B<get_user_dn(uid,dn)>

Returns the dn of an user or undef if no or more then one use was found. 

EXAMPLE:

  my $dn = get_user_dn(varkpete);

  my $dn = get_user_dn(varkpete,dn_of_the_school);

=cut

sub get_user_dn
{
    my $this         = shift;
    my $uid          = shift || return undef;
    my $school_base  = shift || '';
    my $scope        = 'sub';
    my $pref = '';
    my $mesg;

    my $base	= $this->{LDAP_BASE};

    # First we have to search the school prefix;
    if( $school_base ne '' )
    {
      $pref = $this->get_school_config('SCHOOL_LOGIN_PREFIX',$school_base);
      if( $uid !~ /^$pref/ )
      {
        $uid = $pref.$uid;
      }
      $base  = "ou=people,$school_base";
      $scope = 'one';
    }
    
    $mesg = $this->{LDAP}->search( base   => $base,
    			 	   scope  => $scope,
          		           filter => "(&(uid=$uid)(objectClass=SchoolAccount))",
          		           attrs  => [ 'dn' ]
          		);
    if( $mesg->code() || $mesg->count() != 1 )
    {
      return undef;
    }
    return $mesg->entry(0)->dn();
}
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Some helper funktions
#-----------------------------------------------------------------------

1;
